# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import hashlib

from teambition.utils import to_binary
from teambition.api.base import TeambitionAPI


class Webhooks(TeambitionAPI):

    def verify_signature(self, sign, timestamp, nonce):
        """
        校验 webhook 签名

        :param sign: 签名
        :param timestamp: 时间戳
        :param nonce: nonce
        :return: 验证通过返回 True 否则 False
        """
        keys = [
            to_binary(self.client_secret),
            to_binary(timestamp),
            to_binary(nonce),
        ]
        keys.sort()
        keys_str = b''.join(keys)
        signature = hashlib.sha1(keys_str).hexdigest()
        return to_binary(sign) == signature



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import six
from requests import RequestException


class TeambitionException(RequestException):

    def __str__(self):
        if six.PY2:
            return self.response.content
        else:
            return self.response.text

    __repr__ = __str__



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from optionaldict import optionaldict

from teambition.api.base import TeambitionAPI


class Messages(TeambitionAPI):

    def get(self):
        """
        获取消息列表

        详情请参考
        http://docs.teambition.com/wiki/messages#messages-get

        :return: 返回的 JSON 数据包
        """
        return self._get('api/v2/messages')

    def update(self, id, is_archived=None, is_read=None):
        """
        更新消息，标记已读/归档

        详情请参考
        http://docs.teambition.com/wiki/messages#messages-update

        :param id: 消息 ID
        :param is_archived: 可选，是否归档
        :param is_read: 可选，是否已读
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            isArchived=is_archived,
            isRead=is_read
        )
        return self._put(
            'api/messages/{0}'.format(id),
            data=data
        )

    def mark_all_read(self):
        """
        标记所有未读消息为已读

        详情请参考
        http://docs.teambition.com/wiki/messages#messages-update-markallread

        :return: 返回的 JSON 数据包
        """
        return self._put('api/messages/markallread')

    def delete(self, id):
        """
        删除单条消息

        详情请参考
        http://docs.teambition.com/wiki/messages#messages-delete

        :param id: 消息 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/messages/{0}'.format(id))

    def archive_all(self):
        """
        归档所有已读消息

        详情请参考
        http://docs.teambition.com/wiki/messages#messages-delete

        :return: 返回的 JSON 数据包
        """
        return self._delete('api/messages')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from optionaldict import optionaldict

from teambition.api.base import TeambitionAPI


class Posts(TeambitionAPI):

    def get(self, id=None, project_id=None):
        """
        获取分享列表

        详情请参考
        http://docs.teambition.com/wiki/posts#posts-get

        :param id: 可选，分享 ID
        :param project_id: 可选，项目 ID
        :return: 返回的 JSON 数据包
        """
        assert id or project_id

        if id:
            endpoint = 'api/posts/{0}'.format(id)
        elif project_id:
            endpoint = 'api/projects/{0}/posts'.format(project_id)
        return self._get(endpoint)

    def create(self, project_id, title, content, post_mode=None,
               visiable='members', involve_members=None,
               tag_ids=None, attachments=None):
        """
        新建分享

        详情请参考
        http://docs.teambition.com/wiki/posts#posts-create

        :param project_id: 项目 ID
        :param title: 标题
        :param content: 内容
        :param post_mode: 可选，分享模式
        :param visiable: 可选，可见范围，默认为 members，可选 involves
        :param involve_members: 可选，参与者 ID 列表
        :param tag_ids: 可选，标签 ID 列表
        :param attachments: 可选，附件 ID 列表
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            _projectId=project_id,
            title=title,
            content=content,
            postMode=post_mode,
            visiable=visiable,
            involveMembers=involve_members,
            tagIds=tag_ids,
            attachments=attachments
        )
        return self._post(
            'api/posts',
            data=data
        )

    def delete(self, id):
        """
        删除分享

        详情请参考
        http://docs.teambition.com/wiki/posts#posts-delete

        :param id: 分享 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/posts/{0}'.format(id))

    def update(self, id, title=None, content=None, post_mode=None,
               attachments=None, pin=None):
        """
        更新分享

        详情请参考
        http://docs.teambition.com/wiki/posts#posts-update

        :param id: 分享 ID
        :param title: 可选，标题
        :param content: 可选，内容
        :param post_mode: 可选，分享模式
        :param attachments: 可选，附件 ID 列表
        :param pin: 可选，置顶状态，True/False
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            title=title,
            content=content,
            postMode=post_mode,
            attachments=attachments,
            pin=pin
        )
        return self._put(
            'api/posts/{0}'.format(id),
            data=data
        )

    def like(self, id):
        """
        赞分享

        详情请参考
        http://docs.teambition.com/wiki/posts#posts-like

        :param id: 分享 ID
        :return: 返回的 JSON 数据包
        """
        return self._post('api/posts/{0}/like'.format(id))

    def update_members(self, id, members):
        """
        更新分享参与者

        详情请参考
        http://docs.teambition.com/wiki/posts#posts-update-involvemembers

        :param id: 分享 ID
        :param members: 参与者 ID 列表
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/post/{0}'.format(id),
            data={
                'involveMembers': members
            }
        )

    def archive(self, id):
        """
        归档分享

        详情请参考
        http://docs.teambition.com/wiki/posts#posts-archive

        :param id: 分享 ID
        :return: 返回的 JSON 数据包
        """
        return self._post('api/posts/{0}/archive'.format(id))

    def unarchive(self, id):
        """
        取消归档分享

        详情请参考
        http://docs.teambition.com/wiki/posts#posts-unarchive

        :param id: 分享 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/posts/{0}/archive'.format(id))

    def update_tags(self, id, tag_ids):
        """
        更新分享标签

        详情请参考
        http://docs.teambition.com/wiki/posts#posts-update-tags

        :param id: 分享 ID
        :param tags: 标签 ID 列表
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/posts/{0}/tagIds'.format(id),
            data={
                'tagIds': tag_ids
            }
        )

    def get_tags(self, id):
        """
        获取分享标签列表

        :param id: 分享 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/posts/{0}/tags'.format(id))

    def create_tag(self, id, name):
        """
        新建标签

        :param id: 分享 ID
        :param name: 标签名称
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'api/posts/{0}/tags'.format(id),
            data={
                'name': name
            }
        )

    def add_tag(self, id, tag_id):
        """
        关联标签

        :param id: 分享 ID
        :param tag_id: 标签 ID
        :return: 返回的 JSON 数据包
        """
        return self._put('api/posts/{0}/tags/{1}'.format(id, tag_id))

    def get_objectlinks(self, id):
        """
        获取分享关联的 objectlink 列表

        :param id: 分享 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/posts/{0}/objectlinks'.format(id))

    def create_objectlink(self, id, linked_id, linked_type):
        """
        关联对象

        :param id: 分享 ID
        :param linked_id: 关联对象 ID
        :param linked_type: 关联对象类型
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'api/objectlinks',
            data={
                '_parentId': id,
                'parentType': 'post',
                '_linkedId': linked_id,
                'linkedType': linked_type
            }
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from optionaldict import optionaldict

from teambition.api.base import TeambitionAPI


class Organizations(TeambitionAPI):

    def get(self, id=None):
        """
        获取组织信息

        详情请参考
        http://docs.teambition.com/wiki/orgs#orgs-get

        :param id: 可选，组织 ID
        :return: 提供 id 返回特定组织信息，否则返回用户相关组织列表
        """
        if id:
            endpoint = 'api/organizations/{0}'.format(id)
        else:
            endpoint = 'api/organizations'
        return self._get(endpoint)

    def create(self, name, description=None, logo=None,
               location=None, website=None):
        """
        创建新组织

        详情请参考
        http://docs.teambition.com/wiki/orgs#orgs-create

        :param name: 组织名字
        :param description: 可选，描述
        :param logo: 可选，组织 logo
        :param location: 可选，组织所在地
        :param website: 可选，组织网站
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            name=name,
            description=description,
            logo=logo,
            location=location,
            website=website
        )
        return self._post(
            'api/organizations',
            data=data
        )

    def delete(self, id):
        """
        删除组织

        详情请参考
        http://docs.teambition.com/wiki/orgs#orgs-delete

        :param id: 组织 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/organizations/{0}'.format(id))

    def update(self, id, name=None, description=None, logo=None,
               location=None, website=None):
        """
        更新组织信息

        详情请参考
        http://docs.teambition.com/wiki/orgs#orgs-update

        :param id: 组织 ID
        :param name: 组织名字
        :param description: 可选，描述
        :param logo: 可选，组织 logo
        :param location: 可选，组织所在地
        :param website: 可选，组织网站
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            name=name,
            description=description,
            logo=logo,
            location=location,
            website=website
        )
        return self._put(
            'api/organizations/{0}'.format(id),
            data=data
        )

    def update_projects(self, id, project_ids):
        """
        更新组织内项目顺序

        详情请参考
        http://docs.teambition.com/wiki/orgs#orgs-update_projectids

        :param id: 组织 ID
        :param project_ids: 项目 id 列表
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/organizations/{0}/projectIds'.format(id),
            data={
                'projectIds': project_ids
            }
        )

    def add_members(self, id, email):
        """
        添加组织成员

        详情请参考
        http://docs.teambition.com/wiki/orgs#orgs-add-member

        :param id: 组织 ID
        :param email: 邮箱或邮箱列表
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'api/organizations/{0}/members'.format(id),
            data={
                'email': email
            }
        )

    create_members = add_members  # Alias for add_members

    def remove_member(self, id, user_id):
        """
        删除组织成员

        详情请参考
        http://docs.teambition.com/wiki/orgs#orgs-remove-member

        :param id: 组织 ID
        :param user_id: 成员 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete(
            'api/organizations/{0}/members/{1}'.format(id, user_id)
        )

    def get_members(self, id, user_id=None):
        """
        获取组织成员

        详情请参考
        http://docs.teambition.com/wiki/orgs#orgs-get-member

        :param id: 组织 ID
        :param user_id: 可选，成员 ID
        :return: 返回的 JSON 数据包
        """
        if user_id:
            endpoint = 'api/organizations/{0}/members/{1}'.format(id, user_id)
        else:
            endpoint = 'api/organizations/{0}/members'.format(id)
        return self._get(endpoint)

    def get_recommend_members(self, id):
        """
        获取组织推荐成员列表

        详情请参考
        http://docs.teambition.com/wiki/orgs#orgs-recommend-members

        :param id: 组织 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/organizations/{0}/recommendMembers'.format(id))

    def update_member_role(self, id, user_id, role_type):
        """
        更新组织成员角色

        详情请参考
        http://docs.teambition.com/wiki/orgs#orgs-update-member

        :param id: 组织 ID
        :param user_id: 组织成员 ID
        :param role_type: 角色类型, 可选值 member, admin, owner
        """
        return self._put('api/organizations/{0}/members/{1}/{2}'.format(
            id,
            user_id,
            role_type
        ))

    def update_dividers(self, id, dividers):
        """
        更新组织项目分组

        详情请参考
        http://docs.teambition.com/wiki/orgs#orgs-dividers

        :param id: 组织 ID
        :param dividers: 分组列表
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/organizations/{0}/dividers'.format(id),
            data={
                'dividers': dividers
            }
        )

    def get_statistic(self, id, base='member', start_date=None, end_date=None):
        """
        获取组织统计数据

        详情请参参考
        http://docs.teambition.com/wiki/orgs#orgs-statistic

        :param id: 组织 ID
        :param base: 可选，统计基于对象，可选值为 member, team, project，默认为 member
        :param start_date: 可选，开始日期，默认为一个月前
        :param end_date: 可选，结束日期，默认为今天
        :return: 返回的 JSON 数据包
        """
        params = optionaldict(
            base=base,
            startDate=start_date,
            endDate=end_date
        )
        return self._get(
            'api/organizations/{0}/statistic'.format(id),
            params=params
        )

    def get_projects(self, id, project_id=None, is_archived=False):
        """
        获取组织项目

        详情请参考
        http://docs.teambition.com/wiki/orgs#orgs-projects

        :param id: 组织 ID
        :param project_id: 可选，项目 ID
        :param is_archived: 可选，是否归档，默认为 False
        :return: 返回的 JSON 数据包
        """
        if project_id:
            endpoint = 'api/organizations/{0}/projects/{1}'.format(
                id,
                project_id
            )
        else:
            endpoint = 'api/organizations/{0}/projects'
        return self._get(
            endpoint,
            params={
                'isArchived': is_archived
            }
        )

    def quit(self, id):
        """
        退出组织

        详情请参考
        http://docs.teambition.com/wiki/orgs#orgs-quit

        :param id: 组织 ID
        :return: 返回的 JSON 数据包
        """
        return self._put('api/organizations/{0}/quit'.format(id))

    def resend_invitation(self, id, user_id):
        """
        重发邀请邮件

        详情请参考
        http://docs.teambition.com/wiki/orgs#orgs-resend-invitation

        :param id: 组织 ID
        :param user_id: 成员 ID
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/organizations/{0}/members{1}/resend'.format(id, user_id)
        )

    def get_member_tasks(self, id, member_id, start_date=None, is_done=None,
                         all=False, page=1, count=30):
        """
        获取组织成员任务

        详情请参考
        http://docs.teambition.com/wiki/orgs#orgs-get-member-tasks

        :param id: 组织 ID
        :param member_id: 成员 ID
        :param start_date: 可选，起始日期，默认为当周的起始日期
        :param is_done: 可选，是否完成，默认为 False
        :param all: 可选，是否返回所有
        :param page:  可选，页码
        :param count: 可选，每页数量，默认为 30
        :return: 返回的 JSON 数据包
        """
        params = optionaldict(
            startDate=start_date,
            isDone=is_done,
            all=all,
            page=page,
            count=count
        )
        return self._get(
            'api/organizations/{0}/members/{1}/tasks'.format(id, member_id),
            params=params
        )

    def get_member_events(self, id, member_id, start_date=None):
        """
        获取组织成员日程

        详情请参考
        http://docs.teambition.com/wiki/orgs#orgs-get-member-events

        :param id: 组织 ID
        :param member_id: 成员 ID
        :param start_date: 可选，起始日期，默认为当周的起始日期
        :return: 返回的 JSON 数据包
        """
        params = optionaldict(startDate=start_date)
        return self._get(
            'api/organizations/{0}/members/{1}/events'.format(id, member_id),
            params=params
        )

    def get_webhooks(self, id):
        """
        获取组织 Webhook 列表

        :param id: 组织 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/organizations/{0}/hooks'.format(id))

    def create_webhook(self, id, callback_url, active=True, events=None):
        """
        新建组织 webhook

        :param id: 组织 ID
        :param callback_url: 回调地址，Teambition通过HEAD请求进行测试, 有事件被触发将发送POST请求
        :param active: 可选，是否激活，默认为 True
        :param events: 可选，事件列表，默认为空
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            callbackURL=callback_url,
            active=active,
            events=events
        )
        return self._post(
            'api/organizations/{0}/hooks'.format(id),
            data=data
        )

    def update_webhook(self, id, hook_id, callback_url=None, active=True,
                       events=None, add_events=None, remove_events=None):
        """
        更新组织 webhook

        :param id: 组织 ID
        :param hook_id: webhook ID
        :param callback_url: 可选，回调地址，Teambition通过HEAD请求进行测试, 有事件被触发将发送POST请求
        :param active: 可选，是否激活，默认为 True
        :param events: 可选，事件列表，默认为空
        :param add_events: 可选，追加新的事件进去
        :param remove_events: 可选，从原有的事件列表中移除
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            callbackURL=callback_url,
            active=active,
            events=events,
            addEvents=add_events,
            removeEvents=remove_events
        )
        return self._put(
            'api/organizations/{0}/hooks/{1}'.format(id, hook_id),
            data=data
        )

    def delete_webhook(self, id, hook_id):
        """
        删除组织 webhook

        :param id: 组织 ID
        :param hook_id: webhook ID
        :return: 返回的 JSON 数据包
        """
        return self._delete(
            'api/organizations/{0}/hooks/{1}'.format(id, hook_id)
        )

    def get_supported_webhooks(self):
        """
        获取组织支持的 webhook 列表

        :return: 返回的 JSON 数据包
        """
        return self._get('api/organizations/webhooks')

    def get_webhook_value_format(self, event):
        """
        获取组织 webhook 的返回值格式

        :param event: 事件类型
        :return: 返回的 JSON 数据包
        """
        return self._get('api/organizations/webhooks/{0}'.format(event))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals


class TeambitionAPI(object):

    def __init__(self, client=None):
        self._client = client

    def _get(self, url, **kwargs):
        if getattr(self, 'API_BASE_URL', None):
            kwargs['api_base_url'] = self.API_BASE_URL
        return self._client.get(url, **kwargs)

    def _post(self, url, **kwargs):
        if getattr(self, 'API_BASE_URL', None):
            kwargs['api_base_url'] = self.API_BASE_URL
        return self._client.post(url, **kwargs)

    def _put(self, url, **kwargs):
        if getattr(self, 'API_BASE_URL', None):
            kwargs['api_base_url'] = self.API_BASE_URL
        return self._client.put(url, **kwargs)

    def _delete(self, url, **kwargs):
        if getattr(self, 'API_BASE_URL', None):
            kwargs['api_base_url'] = self.API_BASE_URL
        return self._client.delete(url, **kwargs)

    @property
    def access_token(self):
        return self._client.access_token

    @access_token.setter
    def access_token(self, value):
        self._client.access_token = value

    @property
    def client_id(self):
        return self._client.client_id

    @property
    def client_secret(self):
        return self._client.client_secret

    def __str__(self):
        return '<{0}>'.format(self.__class__.__name__)

    __repr__ = __str__



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from optionaldict import optionaldict

from teambition.api.base import TeambitionAPI


class Teams(TeambitionAPI):

    def get(self, id=None, organization_id=None, project_id=None):
        """
        获取团队

        详情请参考
        http://docs.teambition.com/wiki/teams#teams-get

        :param id: 可选，团队 ID
        :param organization_id: 可选，组织 ID
        :param project_id: 可选，项目 ID
        :return: 返回的 JSON 数据包
        """
        if id:
            endpoint = 'api/teams/{0}'.format(id)
        else:
            endpoint = 'api/teams'

        params = optionaldict(
            _organizationId=organization_id,
            _projectId=project_id
        )
        return self._get(endpoint, params=params)

    def create(self, name, organization_id=None):
        """
        新建团队

        详情请参考
        http://docs.teambition.com/wiki/teams#teams-create

        :param name: 团队名称
        :param organization_id: 可选，组织ID
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            name=name,
            _organizationId=organization_id
        )
        return self._post(
            'api/teams',
            data=data
        )

    def delete(self, id):
        """
        删除团队

        详情请参考
        http://docs.teambition.com/wiki/teams#teams-delete

        :param id: 团队 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/teams/{0}'.format(id))

    def update(self, id, name):
        """
        更新团队

        详情请参考
        http://docs.teambition.com/wiki/teams#teams-update

        :param id: 团队 ID
        :param name: 团队名称
        """
        return self._put(
            'api/teams/{0}'.format(id),
            data={
                'name': name
            }
        )

    def bind_project(self, id, project_id):
        """
        关联团队与项目

        详情请参考
        http://docs.teambition.com/wiki/teams#teams-bind-project

        :param id: 团队 ID
        :param project_id: 项目 ID
        :return: 返回的 JSON 数据包
        """
        return self._put('api/teams/{0}/projects/{1}'.format(id, project_id))

    def unbind_project(self, id, project_id):
        """
        取消关联团队与项目

        详情请参考
        http://docs.teambition.com/wiki/teams#teams-unbind-project

        :param id: 团队 ID
        :param project_id: 项目 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete(
            'api/teams/{0}/projects/{1}'.format(id, project_id)
        )

    def add_members(self, id, email):
        """
        添加团队成员

        详情请参考
        http://docs.teambition.com/wiki/teams#teams-add-member

        :param id: 团队 ID
        :param email: 邮箱或邮箱列表
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'api/teams/{0}/members'.format(id),
            data={
                'email': email
            }
        )

    create_members = add_members

    def remove_member(self, id, user_id):
        """
        删除团队成员

        详情请参考
        http://docs.teambition.com/wiki/teams#teams-remove-member

        :param id: 团队 ID
        :param user_id: 成员 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete(
            'api/teams/{0}/members/{1}'.format(id, user_id)
        )

    def get_members(self, id, user_id=None):
        """
        获取团队成员

        详情请参考
        http://docs.teambition.com/wiki/teams#teams-get-member

        :param id: 团队 ID
        :param user_id: 可选，用户 ID
        :return: 返回的 JSON 数据包
        """
        if user_id:
            endpoint = 'api/teams/{0}/members/{1}'.format(id, user_id)
        else:
            endpoint = 'api/teams/{0}/members'.format(id)
        return self._get(endpoint)

    def quit(self, id):
        """
        退出团队

        详情请参考
        http://docs.teambition.com/wiki/teams#teams-quit

        :param id: 团队 ID
        :return: 返回的 JSON 数据包
        """
        return self._put('api/teams/{0}/quit'.format(id))

    def get_memeber_tasks(self, id, member_id, start_date=None):
        """
        获取团队成员任务

        详情请参考
        http://docs.teambition.com/wiki/teams#teams-get-team-member-tasks

        :param id: 团队 ID
        :param member_id: 成员 ID
        :param start_date: 可选，起始日期，默认为当周的起始日期
        :return: 返回的 JSON 数据包
        """
        params = optionaldict(startDate=start_date)
        return self._get(
            'api/teams/{0}/members/{1}/tasks'.format(id, member_id),
            params=params
        )

    def get_member_events(self, id, member_id, start_date=None):
        """
        获取团队成员日程

        详情请参考
        http://docs.teambition.com/wiki/teams#teams-get-team-member-events

        :param id: 团队 ID
        :param member_id: 成员 ID
        :param start_date: 可选，起始日期，默认为当周的起始日期
        :return: 返回的 JSON 数据包
        """
        params = optionaldict(startDate=start_date)
        return self._get(
            'api/teams/{0}/members/{1}/events'.format(id, member_id),
            params=params
        )

    def get_tasks(self, id, start_date=None):
        """
        获取团队周任务

        详情请参考
        http://docs.teambition.com/wiki/teams#teams-get-team-week-tasks

        :param id: 团队 ID
        :param start_date: 可选，起始日期，默认为当周的起始日期
        :return: 返回的 JSON 数据包
        """
        params = optionaldict(startDate=start_date)
        return self._get(
            'api/teams/{0}/tasks'.format(id),
            params=params
        )

    def get_events(self, id, start_date=None):
        """
        获取团队周日程

        详情请参考
        http://docs.teambition.com/wiki/teams#teams-get-team-week-events

        :param id: 团队 ID
        :param start_date: 可选，起始日期，默认为当周的起始日期
        :return: 返回的 JSON 数据包
        """
        params = optionaldict(startDate=start_date)
        return self._get(
            'api/teams/{0}/events'.format(id),
            params=params
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from optionaldict import optionaldict

from teambition.api.base import TeambitionAPI


class Works(TeambitionAPI):

    def get(self, id=None, parent_id=None, page=None, count=None, all=None):
        """
        获取文件信息

        详情请参考
        http://docs.teambition.com/wiki/works#works-get

        :param id: 可选，文件 ID
        :param parent_id: 可选，父级 ID
        :param page: 可选，当前页，默认为 1
        :param count: 可选，每页数量，默认为 30
        :param all: 可选，若提供此参数则返回所有
        :return: 返回的 JSON 数据包
        """
        assert id or parent_id

        params = optionaldict(
            page=page,
            count=count,
            all=all
        )
        if id:
            endpoint = 'api/works/{0}'.format(id)
        elif parent_id:
            endpoint = 'api/works'
            params['_parentId'] = parent_id
        return self._get(endpoint, params=params)

    def create(self, parent_id, file_name, file_size, file_type, file_category,
               file_key, image_width=None, image_height=None,
               involve_members=None):
        """
        新建文件

        详情请参考
        http://docs.teambition.com/wiki/works#works-create

        :param parent_id: 所属目录 ID
        :param file_name: 文件名
        :param file_size: 文件大小
        :param file_type: 文件类型
        :param file_category: 文件类别
        :param file_key: 使用 striker 服务上传后可得
        :param image_width: 可选，图片宽度
        :param image_height: 可选，图片高度
        :param involve_members: 可选
        """
        data = optionaldict(
            _parentId=parent_id,
            fileName=file_name,
            fileSize=file_size,
            fileType=file_type,
            fileCategory=file_category,
            fileKey=file_key,
            imageWidth=image_width,
            imageHeight=image_height,
            involveMembers=involve_members
        )
        return self._post(
            'api/works',
            data=data
        )

    def like(self, id):
        """
        赞文件

        详情请参考
        http://docs.teambition.com/wiki/works#works-like

        :param id: 文件 ID
        :return: 返回的 JSON 数据包
        """
        return self._post('api/works/{0}/like'.format(id))

    def update(self, id, file_name, description=None):
        """
        更新文件

        详情请参考
        http://docs.teambition.com/wiki/works#works-update

        :param id: 文件 ID
        :param file_name: 文件名
        :param description: 可选，描述
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            fileName=file_name,
            description=description
        )
        return self._put(
            'api/works/{0}'.format(id),
            data=data
        )

    def move(self, id, parent_id):
        """
        移动文件

        详情请参考
        http://docs.teambition.com/wiki/works#works-move

        :param id: 文件 ID
        :param parent_id: 新的目录 ID
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/works/{0}'.format(id),
            data={
                '_parentId': parent_id
            }
        )

    def delete(self, id):
        """
        删除文件

        详情请参考
        http://docs.teambition.com/wiki/works#works-delete

        :param id: 文件 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/works/{0}'.format(id))

    def update_members(self, id, members):
        """
        更新文件参与者

        详情请参考
        http://docs.teambition.com/wiki/works#works-update-involvemembers

        :param id: 文件 ID
        :param members: 参与者 ID 列表
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/works/{0}/involveMembers'.format(id),
            data={
                'involveMembers': members
            }
        )

    def get_tags(self, id):
        """
        获取任务标签列表

        :param id: 文件 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/works/{0}/tags'.format(id))

    def create_tag(self, id, name):
        """
        新建标签

        :param id: 文件 ID
        :param name: 标签名称
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'api/works/{0}/tags'.format(id),
            data={
                'name': name
            }
        )

    def add_tag(self, id, tag_id):
        """
        关联标签

        :param id: 文件 ID
        :param tag_id: 标签 ID
        :return: 返回的 JSON 数据包
        """
        return self._put('api/works/{0}/tags/{1}'.format(id, tag_id))

    def get_objectlinks(self, id):
        """
        获取文件关联的 objectlink 列表

        :param id: 文件 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/works/{0}/objectlinks'.format(id))

    def create_objectlink(self, id, linked_id, linked_type):
        """
        关联对象

        :param id: 文件 ID
        :param linked_id: 关联对象 ID
        :param linked_type: 关联对象类型
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'api/objectlinks',
            data={
                '_parentId': id,
                'parentType': 'work',
                '_linkedId': linked_id,
                'linkedType': linked_type
            }
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from datetime import datetime

import pytz
from optionaldict import optionaldict

from teambition.api.base import TeambitionAPI


class Projects(TeambitionAPI):

    def get(self, id=None, team_id=None, is_archived=False):
        """
        获取项目信息

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-get

        :param id: 可选，不提供则返回用户所在的项目
        :param team_id: 可选，团队 ID，提供可获取团队所在项目
        :param is_archived: 返回归档的项目，默认为 False
        :return: 返回的 JSON 数据包
        """
        params = optionaldict(
            _teamId=team_id,
            isArchived=is_archived
        )
        if id is None:
            endpoint = 'api/projects'
        else:
            endpoint = 'api/projects/{0}'.format(id)

        return self._get(
            endpoint,
            params=params
        )

    def create(self, name, description=None, logo=None, categroy=None,
               divider_index=None, organization_id=None):
        """
        新建项目

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-create

        :param name: 项目名称
        :param description: 可选，项目描述
        :param logo: 可选，项目 logo
        :param category: 可选，项目类别
        :param divider_index: 可选，dividers 中得 index，仅对拥有者有效
        :param organization_id: 可选，组织 ID，创建组织项目时需要提供此参数
        :return: 新建的项目信息
        """
        data = optionaldict(
            name=name,
            description=description,
            logo=logo,
            category=categroy,
            dividerIndex=divider_index,
            _organizationId=organization_id
        )
        return self._post(
            'api/projects',
            data=data
        )

    def delete(self, id):
        """
        删除项目

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-delete

        :param id: 路径参数
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/projects/{0}'.format(id))

    def update(self, id, name=None, description=None, logo=None,
               category=None, is_archived=None, is_public=None):
        """
        更新项目

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-update

        :param id: 路径参数
        :param name: 项目名称
        :param description: 可选，项目描述
        :param logo: 可选，项目 logo
        :param category: 可选，项目类别
        :param is_archived: 可选，是否归档
        :param is_public: 是否公开，需要拥有者权限
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            name=name,
            description=description,
            logo=logo,
            category=category,
            isArchived=is_archived,
            isPublic=is_public
        )
        return self._put(
            'api/projects/{0}'.format(id),
            data=data
        )

    def star(self, id):
        """
        项目加星

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-star

        :param id: 路径参数
        """
        return self._put('api/projects/{0}/star'.format(id))

    def unstar(self, id):
        """
        取消项目加星

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-star

        :param id: 路径参数
        """
        return self._delete('api/projects/{0}/star'.format(id))

    def copy(self, id, name, organization_id=None, is_public=False):
        """
        复制项目

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-copy

        :param id: 路径参数
        :param name: 新项目名称
        :param organization_id: 可选，所属组织 ID，若为空则属于个人
        :param is_public: 可选，是否公开，默认为 False
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            name=name,
            _organizationId=organization_id,
            isPublic=is_public
        )
        return self._put(
            'api/projects/{0}/copy'.format(id),
            data=data
        )

    def get_posts(self, id):
        """
        获取项目分享列表

        :param id: 项目 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/projects/{0}/posts'.format(id))

    def get_tags(self, id):
        """
        获取项目标签列表

        :param id: 项目 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/projects/{0}/tags'.format(id))

    def update_navigation(self, id, navigation):
        """
        更新项目应用导航

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-update-apps

        :param id: 项目 ID
        :param navigation: 字典，新的应用顺序
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/projects/{0}/navigation'.format(id),
            data={
                'navigation': navigation
            }
        )

    def add_members(self, id, email):
        """
        添加项目成员

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-add-member

        :param id: 项目 ID
        :param email: 邮箱或邮箱列表
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'api/projects/{0}/members'.format(id),
            data={
                'email': email
            }
        )

    create_members = add_members  # Alias for add_members

    def remove_member(self, id, user_id):
        """
        删除项目成员

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-remove-member

        :param id: 项目 ID
        :param user_id: 成员 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete(
            'api/projects/{0}/members/{1}'.format(id, user_id)
        )

    def get_members(self, id, user_id=None):
        """
        获取项目成员

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-get-members

        :param id: 项目 ID
        :param user_id: 可选，成员 ID
        :return: 返回的 JSON 数据包
        """
        if user_id:
            endpoint = 'api/projects/{0}/members/{1}'.format(id, user_id)
        else:
            endpoint = 'api/projects/{0}/members'.format(id)
        return self._get(endpoint)

    def get_recommend_members(self, id):
        """
        获取项目推荐成员列表

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-recommend-members

        :param id: 项目 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/projects/{0}/recommendMembers'.format(id))

    def update_member_role(self, id, user_id, role_type):
        """
        更新项目成员角色

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-update-member

        :param id: 项目 ID
        :param user_id: 成员 ID
        :param role_type: 角色类型, 可选值 member, admin, owner
        :return: 返回的 JSON  数据包
        """
        return self._put('api/projects/{0}/members/{1}/{2}'.format(
            id,
            user_id,
            role_type
        ))

    def transfer(self, id, organization_id=None):
        """
        移交项目

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-transfer

        :param id: 项目 ID
        :param organization_id: 目的组织 ID
        """
        return self._put(
            'api/projects/{0}/transfer'.format(id),
            data={
                '_organizationId': organization_id
            }
        )

    def get_statistic(self, id, today=True):
        """
        获取项目统计数据

        详情请参参考
        http://docs.teambition.com/wiki/projects#projects-statistic

        :param id: 项目 ID
        :param today: 可选，默认为今天，以该日期为准，退后五天内每天的任务完成数
        :return: 返回的 JSON 数据包
        """
        return self._get(
            'api/projects/{0}/statistic'.format(id),
            params={
                'today': today
            }
        )

    def quit(self, id):
        """
        退出项目

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-quit

        :param id: 项目 ID
        :return: 返回的 JSON 数据包
        """
        return self._put('api/projects/{0}/quit'.format(id))

    def resend_invitation(self, id, user_id):
        """
        重发邀请邮件

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-resend-invitation

        :param id: 项目 ID
        :param user_id: 成员 ID
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/projects/{0}/members{1}/resend'.format(id, user_id)
        )

    def reset_invitation(self, id):
        """
        重置项目邀请链接

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-reset-invitelink

        :param id: 项目 ID
        :return: 返回的 JSON 数据包
        """
        return self._put('api/projects/{0}/invitelink'.format(id))

    def update_tasklists(self, id, tasklist_ids):
        """
        自定义项目内任务分组排序

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-update-tasklistIds

        :param id: 项目 ID
        :param tasklist_ids: 所有未归档的任务分组 ID 列表
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/projects/{0}/tasklistIds',
            data={
                'tasklistIds': tasklist_ids
            }
        )

    def get_activities(self, id, start_date=None, end_date=None, limit=30):
        """
        获取主页动态

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-home-activities

        :param id: 项目 ID
        :param start_date: 可选，开始时间，默认为当天的开始
        :param end_date: 可选，结束时间，默认为当天的 23 时 59 分 59 秒
        :param limit: 可选，数量限制，默认为 30
        :return: 返回的 JSON 数据包
        """
        params = optionaldict(
            startDate=start_date,
            endDate=end_date,
            limit=limit
        )
        return self._get(
            'api/projects/{0}/activities'.format(id),
            params=params
        )

    def get_reviews(self, id, start_date=None, end_date=None):
        """
        获取项目历史动态

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-reviews

        :param id: 项目 ID
        :param start_date: 可选，开始时间，默认为当天的开始
        :param end_date: 可选，结束时间，默认为当天的 23 时 59 分 59 秒
        :return: 返回的 JSON 数据包
        """
        params = optionaldict(
            startDate=start_date,
            endDate=end_date
        )
        return self._get(
            'api/projects/{0}/reviews'.format(id),
            params=params
        )

    def get_tasks(self, id=None, executor_id=None, fields=None,
                  subtask_fields=None, with_tasklist=False, with_subtasks=False,
                  with_tags=False, with_executor=False, is_done=False, all=False,
                  page=1, limit=30, **kwargs):
        """
        获取项目任务

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-get-tasks

        :param id: 可选，项目 ID
        :param executor_id: 可选，执行者 ID
        :param field: 可选，属性列表, 如: _id,_projectId,isDone,content
        :param subtask_fields: 可选，子任务属性列表，有: _id, _taskId, _executorId, content, order
        :param with_tasklist: 可选，是否包含任务分组信息，默认为 False
        :param with_subtasks: 可选，是否包含子任务信息，默认为 False
        :param with_tags: 可选，是否包含标签信息，默认为 False
        :param with_executor: 可选，是否包含执行者信息，默认为 False
        :param is_done: 可选，是否完成，默认为 False
        :param all: 可选，是否包含所有类型，包括完成与未完成，默认为 False
        :param page: 可选，页码，默认为 1
        :param limit: 可选，每页数量，默认为 30 ，最大为 1000
        :return: 返回的 JSON 数据包
        """
        if id:
            endpoint = 'api/projects/{0}/tasks'.format(id)
        else:
            endpoint = 'api/projects/tasks'

        if isinstance(fields, (tuple, list)):
            fields = ','.join(fields)
        if isinstance(subtask_fields, (tuple, list)):
            subtask_fields = ','.join(subtask_fields)

        params = optionaldict(
            _executorId=executor_id,
            fields=fields,
            subtaskFields=subtask_fields,
            withTasklist=with_tasklist,
            withSubtasks=with_subtasks,
            withTags=with_tags,
            withExecutor=with_executor,
            isDone=is_done,
            all=all,
            page=page,
            limit=limit,
            **kwargs
        )
        return self._get(endpoint, params=params)

    def get_events(self, id=None, start_date=None, end_date=None):
        """
        获取项目日程

        详情请参考
        http://docs.teambition.com/wiki/projects#projects-get-events

        :param id: 可选，项目 ID
        :param start_date: 可选，起始日期
        :param end_date: 可选，截止日期
        :return: 返回的 JSON 数据包
        """
        start_date = start_date or datetime.utcnow().replace(tzinfo=pytz.utc)
        if id:
            endpoint = 'api/projects/{0}/events'.format(id)
        else:
            endpoint = 'api/projects/events'
        params = optionaldict(
            start_date=start_date,
            end_date=end_date
        )
        return self._get(endpoint, params=params)

    def get_tasklists(self, id):
        """
        获取项目任务分组列表

        详情请参考
        http://docs.teambition.com/wiki/tasklists#tasklists-get

        :param id: 项目 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/projects/{0}/tasklists'.format(id))

    def get_webhooks(self, id):
        """
        获取项目 Webhook 列表

        :param id: 项目 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/projects/{0}/hooks'.format(id))

    def create_webhook(self, id, callback_url, active=True, events=None):
        """
        新建项目 webhook

        :param id: 项目 ID
        :param callback_url: 回调地址，Teambition通过HEAD请求进行测试, 有事件被触发将发送POST请求
        :param active: 可选，是否激活，默认为 True
        :param events: 可选，事件列表，默认为空
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            callbackURL=callback_url,
            active=active,
            events=events
        )
        return self._post(
            'api/projects/{0}/hooks'.format(id),
            data=data
        )

    def update_webhook(self, id, hook_id, callback_url=None, active=True,
                       events=None, add_events=None, remove_events=None):
        """
        更新项目 webhook

        :param id: 项目 ID
        :param hook_id: webhook ID
        :param callback_url: 可选，回调地址，Teambition通过HEAD请求进行测试, 有事件被触发将发送POST请求
        :param active: 可选，是否激活，默认为 True
        :param events: 可选，事件列表，默认为空
        :param add_events: 可选，追加新的事件进去
        :param remove_events: 可选，从原有的事件列表中移除
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            callbackURL=callback_url,
            active=active,
            events=events,
            addEvents=add_events,
            removeEvents=remove_events
        )
        return self._put(
            'api/projects/{0}/hooks/{1}'.format(id, hook_id),
            data=data
        )

    def delete_webhook(self, id, hook_id):
        """
        删除项目 webhook

        :param id: 项目 ID
        :param hook_id: webhook ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/projects/{0}/hooks/{1}'.format(id, hook_id))

    def get_supported_webhooks(self):
        """
        获取项目支持的 webhook 列表

        :return: 返回的 JSON 数据包
        """
        return self._get('api/projects/webhooks')

    def get_webhook_value_format(self, event):
        """
        获取项目 webhook 的返回值格式

        :param event: 事件类型
        :return: 返回的 JSON 数据包
        """
        return self._get('api/projects/webhooks/{0}'.format(event))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from optionaldict import optionaldict

from teambition.api.base import TeambitionAPI


class Activities(TeambitionAPI):

    def create(self, content, object_id, object_type,
               attachments=None, mentions=None):
        """
        新建评论

        详情请参考
        http://docs.teambition.com/wiki/activities#activities-comment

        :param content: 内容
        :param object_id: 所属对象 ID
        :param object_type: 所属对象类型，目前仅支持评论: post/task/event/entry/work
        :param attachments: 可选，附件 ID 列表
        :param mentions: 可选，提及，格式为 {"user id": "@username"}
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            content=content,
            _boundToObjectId=object_id,
            boundToObjectType=object_type,
            attachments=attachments,
            mentions=mentions
        )
        return self._post(
            'api/activities',
            data=data
        )

    def update(self, id, content):
        """
        更新评论

        详情请参考
        http://docs.teambition.com/wiki/activities#activities-update_comment

        :param id: 动态 ID
        :param content: 内容
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/activities/{0}'.format(id),
            data={
                'content': content
            }
        )

    def delete(self, id):
        """
        删除评论

        详情请参考
        http://docs.teambition.com/wiki/activities#activities-delete

        :param id: 动态 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/activities/{0}'.format(id))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import copy
try:
    import simplejson as json
except ImportError:
    import json

import six
import requests

from teambition import api
from teambition.api.base import TeambitionAPI
from teambition.utils import JSONEncoder, JSONDecoder
from teambition.exceptions import TeambitionException


class Teambition(object):
    """
    Teambition API 客户端
    """

    API_BASE_URL = 'https://api.teambition.com/'

    # API endpoints
    oauth = api.OAuth()
    """:doc:`oauth`"""
    projects = api.Projects()
    """:doc:`projects`"""
    tasklists = api.Tasklists()
    """:doc:`tasklists`"""
    stages = api.Stages()
    """:doc:`stages`"""
    tasks = api.Tasks()
    """:doc:`tasks`"""
    users = api.Users()
    """:doc:`users`"""
    organizations = api.Organizations()
    """:doc:`organizations`"""
    stagetemplates = api.StageTemplates()
    """:doc:`stagetemplates`"""
    teams = api.Teams()
    """:doc:`teams`"""
    subtasks = api.Subtasks()
    """:doc:`subtasks`"""
    messages = api.Messages()
    """:doc:`messages`"""
    posts = api.Posts()
    """:doc:`posts`"""
    collections = api.Collections()
    """:doc:`collections`"""
    works = api.Works()
    """:doc:`works`"""
    events = api.Events()
    """:doc:`events`"""
    tags = api.Tags()
    """:doc:`tags`"""
    objectlinks = api.ObjectLinks()
    """:doc:`objectlinks`"""
    activities = api.Activities()
    """:doc:`activities`"""
    webhooks = api.Webhooks()
    """:doc:`webhooks`"""

    def __new__(cls, *args, **kwargs):
        self = super(Teambition, cls).__new__(cls)
        for name, api in self.__class__.__dict__.items():
            if isinstance(api, TeambitionAPI):
                api = copy.deepcopy(api)
                api._client = self
                setattr(self, name, api)
        return self

    def __init__(self, client_id, client_secret, access_token=None):
        """
        初始化 Teambition API Client

        :param client_id: 申请应用时分配的 client_id
        :param client_secret: 申请应用时分配的 client_secret
        :param access_token: 可选，access_token
        """
        self.client_id = client_id
        self.client_secret = client_secret
        self._access_token = access_token

    def _request(self, method, endpoint, **kwargs):
        if not endpoint.startswith(('http://', 'https://')):
            api_base_url = kwargs.pop('api_base_url', self.API_BASE_URL)
            url = '{base}{endpoint}'.format(
                base=api_base_url,
                endpoint=endpoint
            )
        else:
            url = endpoint

        if isinstance(kwargs.get('data', ''), dict):
            body = json.dumps(
                kwargs['data'],
                ensure_ascii=False,
                cls=JSONEncoder
            )
            body = body.encode('utf-8')
            kwargs['data'] = body

        if 'headers' not in kwargs:
            kwargs['headers'] = {}
        if 'Content-Type' not in kwargs['headers']:
            kwargs['headers']['Content-Type'] = 'application/json'
        if 'Authorization' not in kwargs['headers']:
            kwargs['headers']['Authorization'] = 'OAuth2 {0}'.format(
                self.access_token
            )

        res = requests.request(
            method=method,
            url=url,
            **kwargs
        )
        try:
            res.raise_for_status()
        except requests.RequestException as reqe:
            raise TeambitionException(
                request=reqe.request,
                response=reqe.response
            )

        result = res.json(cls=JSONDecoder)
        return result

    def get(self, endpoint, **kwargs):
        return self._request('get', endpoint, **kwargs)

    def post(self, endpoint, **kwargs):
        return self._request('post', endpoint, **kwargs)

    def put(self, endpoint, **kwargs):
        return self._request('put', endpoint, **kwargs)

    def delete(self, endpoint, **kwargs):
        return self._request('delete', endpoint, **kwargs)

    @property
    def access_token(self):
        """
        获取 access_token

        :return: access_token
        """
        return self._access_token

    @access_token.setter
    def access_token(self, token):
        """
        设置 access_token

        :param token: access_token
        """
        self._access_token = token



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import datetime
try:
    import simplejson as json
except ImportError:
    import json

import six
import dateutil.parser


class JSONEncoder(json.JSONEncoder):

    def default(self, obj):
        if isinstance(obj, datetime.date):
            encoded = obj.isoformat()
        elif isinstance(obj, datetime.datetime):
            encoded = obj.isoformat()
        else:
            encoded = super(JSONEncoder, self).default(obj)
        return encoded


class JSONDecoder(json.JSONDecoder):

    def __init__(self, *args, **kwargs):
        kwargs[b'object_hook'] = self.datetime_object_hook
        super(JSONDecoder, self).__init__(*args, **kwargs)

    def datetime_object_hook(self, obj):
        for key, value in six.iteritems(obj):
            if not isinstance(value, six.string_types):
                continue
            # TODO: a better way to identify ISO datetime
            if len(value) not in (10, 18, 20, 22, 24, 25, 26):
                continue
            try:
                value = dateutil.parser.parse(value)
            except (TypeError, ValueError):
                continue
            obj[key] = value
        return obj


def to_text(value, encoding='utf-8'):
    """Convert value to unicode, default encoding is utf-8

    :param value: Value to be converted
    :param encoding: Desired encoding
    """
    if not value:
        return ''
    if isinstance(value, six.text_type):
        return value
    if isinstance(value, six.binary_type):
        return value.decode(encoding)
    return six.text_type(value)


def to_binary(value, encoding='utf-8'):
    """Convert value to binary string, default encoding is utf-8

    :param value: Value to be converted
    :param encoding: Desired encoding
    """
    if not value:
        return b''
    if isinstance(value, six.binary_type):
        return value
    if isinstance(value, six.text_type):
        return value.encode(encoding)
    return six.binary_type(value)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from optionaldict import optionaldict

from teambition.api.base import TeambitionAPI


class Users(TeambitionAPI):

    def me(self):
        """
        获取个人信息

        详情请参考
        http://docs.teambition.com/wiki/me#me-get

        :return: 返回的 JSON 数据包
        """
        return self._get('api/users/me')

    def update(self, name=None, avatar_url=None, title=None, birthday=None,
               location=None, phone=None, website=None):
        """
        更新个人信息

        详情请参考
        http://docs.teambition.com/wiki/me#me-update-info

        :param name: 可选，姓名
        :param avatar_url: 可选，头像地址
        :param title: 可选，职位
        :param birthday: 可选，出生日期
        :param location: 可选，所在地
        :param phone: 可选，手机号码
        :param website: 可选，个人站点
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            name=name,
            avatarUrl=avatar_url,
            title=title,
            birthday=birthday,
            location=location,
            phone=phone,
            website=website
        )
        return self._put(
            'api/users',
            data=data
        )

    def add_email(self, email):
        """
        添加新邮箱

        详情请参考
        http://docs.teambition.com/wiki/me#me-update-emails

        :param email: 邮箱地址
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'api/users/email',
            data={
                'email': email
            }
        )

    def update_preferences(self, id, notification):
        """
        更新通知设置

        详情请参考
        http://docs.teambition.com/wiki/me#me-update-notification

        :param id: 用户 ID
        :param notification: 新通知设置，dict 类型
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/preferences/{0}'.format(id),
            data={
                'notification': notification
            }
        )

    def send_verification_email(self, email_id):
        """
        发送邮箱验证邮件

        详情请参考
        http://docs.teambition.com/wiki/me#me-send-verify-email

        :param email_id: 邮箱唯一标识 ID，可以从 emails 属性中获得
        :return: 返回的 JSON 数据包
        """
        return self._post('api/users/email/{0}/send'.format(email_id))

    def delete_email(self, email_id):
        """
        删除关联邮箱

        详情请参考
        http://docs.teambition.com/wiki/me#me-delete-email

        :param email_id: 邮箱唯一标识 ID，可以从 emails 属性中获得
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/users/email/{0}/send'.format(email_id))

    def get_my_tasks(self, page=1, count=30):
        """
        获取执行者为我的任务

        详情请参考
        http://docs.teambition.com/wiki/me#me-tasks

        :param page: 可选，当前页码
        :param count: 可选，每页数量，默认为 30
        :return: 返回的 JSON 数据包
        """
        return self._get('api/tasks/me')

    def get_involved_tasks(self, page=1, count=30):
        """
        获取我参与的任务

        详情请参考
        http://docs.teambition.com/wiki/me#me-involves

        :param page: 可选，当前页码
        :param count: 可选，每页数量，默认为 30
        :return: 返回的 JSON 数据包
        """
        return self._get('api/tasks/involves')

    def get_today_count(self):
        """
        获取今日待处理事项总数

        详情请参考
        http://docs.teambition.com/wiki/me#me-today

        :return: 今日待处理事项数量
        """
        res = self._get('api/users/todayCount')
        return res['todayCount']



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from optionaldict import optionaldict

from teambition.api.base import TeambitionAPI


class Stages(TeambitionAPI):

    def get(self, tasklist_id):
        """
        获取任务分组的阶段列表

        详情请参考
        http://docs.teambition.com/wiki/stages#stages-get

        :param tasklist_id: 任务分组 ID
        :return: 返回的 JSON 数据包
        """
        return self._get(
            'api/stages',
            params={
                '_tasklistId': tasklist_id
            }
        )

    def create(self, name, tasklist_id, prev_id):
        """
        新建阶段

        详情请参考
        http://docs.teambition.com/wiki/stages#stages-create

        :param name: 阶段名称
        :param tasklist_id: 任务分组 ID
        :param prev_id: 前一个阶段的 ID
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'api/stages',
            data={
                'name': name,
                '_tasklistId': tasklist_id,
                '_prevId': prev_id
            }
        )

    def delete(self, id):
        """
        删除阶段（流程模式下，最后一个阶段无法被删除）

        详情请参考
        http://docs.teambition.com/wiki/stages#stages-delete

        :param id: 阶段 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/stages/{0}'.format(id))

    def update(self, id, name=None, is_locked=None):
        """
        更新阶段

        详情请参考
        http://docs.teambition.com/wiki/stages#stages-update

        :param id: 阶段 ID
        :param name: 阶段名称
        :param is_locked: 锁定最后一个阶段
        """
        data = optionaldict(
            name=name,
            isLocked=is_locked
        )
        return self._put(
            'api/stages/{0}'.format(id),
            data=data
        )

    def get_tasks(self, id, executor_id=None, is_done=False, all=False,
                  page=1, limit=30, **kwargs):
        """
        获取阶段下得任务列表

        详情请参考
        http://docs.teambition.com/wiki/stages#stages-get-tasks

        :param id: 阶段 ID
        :param executor_id: 可选，执行者 ID
        :param is_done: 可选，是否已完成，默认为 False
        :param all: 可选，所有类型，包括完成与未完成
        :param page: 可选，页码，默认为 1
        :param limit: 可选，每页数量，默认为 30，最大值为 1000
        :return: 返回的 JSON 数据包
        """
        params = optionaldict(
            isDone=is_done,
            _executorId=executor_id,
            all=all,
            page=page,
            limit=limit,
            **kwargs
        )
        return self._get(
            'api/stages/{0}/tasks'.format(id),
            params=params
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from teambition.api.base import TeambitionAPI


class ObjectLinks(TeambitionAPI):

    def get(self, parent_id, parent_type):
        """
        获取对象关联的 objectlink 列表

        详情请参考
        http://docs.teambition.com/wiki/objectlinks#objectlinks-get

        :param parent_id: 主体对象 ID
        :param parent_type: 主体对象类型
        :return: 返回的 JSON 数据包
        """
        return self._get(
            'api/objectlinks',
            params={
                '_parentId': parent_id,
                'parentType': parent_type
            }
        )

    def create(self, parent_id, parent_type, linked_id, linked_type):
        """
        新建关联

        详情请参考
        http://docs.teambition.com/wiki/objectlinks#objectlinks-create

        :param parent_id: 主体对象 ID
        :param parent_type: 主体对象类型
        :param linked_id: 关联对象 ID
        :param linked_type: 关联对象类型
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'api/objectlinks',
            data={
                '_parentId': parent_id,
                'parentType': parent_type,
                '_linkedId': linked_id,
                'linkedType': linked_type
            }
        )

    def delete(self, id):
        """
        删除关联

        详情请参考
        http://docs.teambition.com/wiki/objectlinks#objectlinks-delete

        :param id: 关联 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/objectlinks/{0}'.format(id))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from teambition.client import Teambition  # NOQA
from teambition.exceptions import TeambitionException  # NOQA


__all__ = ['Teambition', 'TeambitionException']

__author__ = 'messense'
__version__ = '0.1.6'



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from optionaldict import optionaldict

from teambition.api.base import TeambitionAPI


class Events(TeambitionAPI):

    def get(self, id=None, project_id=None):
        """
        获取日程

        详情请参考
        http://docs.teambition.com/wiki/events#events-get

        :param id: 可选，日程 ID
        :param project_id: 可选，项目 ID
        :return: 返回的 JSON 数据包
        """
        endpoint = 'api/events'
        if id:
            endpoint = 'api/events/{0}'.format(id)
        elif project_id:
            endpoint = 'api/projects/{0}/events'.format(project_id)
        return self._get(endpoint)

    def create(self, project_id, title, start_date, end_date, location=None,
               status=None, reminders=None, content=None, recurrence=None,
               source_id=None, mode=None, visiable='members', tag_ids=None):
        """
        新建日程

        详情请参考
        http://docs.teambition.com/wiki/events#events-create

        :param project_id: 项目 ID
        :param title: 内容
        :param start_date: 开始时间
        :param end_date: 结束时间
        :param location: 可选，地点
        :param status: 可选，状态
        :param reminders: 可选，提醒
        :param content: 可选，备注
        :param recurrence: 可选，重复日程规则列表
        :param source_id: 可选，需要派生的重复日程
        :param mode: 可选，新建模式，可选值为 single, after
        :param visiable: 可选，可见范围，默认为 members，可选 involves
        :param tag_ids: 可选，标签 ID 列表
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            _projectId=project_id,
            title=title,
            startDate=start_date,
            endDate=end_date,
            location=location,
            status=status,
            reminders=reminders,
            content=content,
            recurrence=recurrence,
            _sourceId=source_id,
            mode=mode,
            visiable=visiable,
            tagIds=tag_ids
        )
        return self._post(
            'api/events',
            data=data
        )

    def delete(self, id):
        """
        删除日程

        详情请参考
        http://docs.teambition.com/wiki/events#events-delete

        :param id: 日程 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/events/{0}'.format(id))

    def update(self, id, title=None, start_date=None, end_date=None,
               location=None, status=None, reminders=None, content=None):
        """
        更新日程

        详情请参考
        http://docs.teambition.com/wiki/events#events-update

        :param id: 日程 ID
        :param title: 内容
        :param start_date: 开始时间
        :param end_date: 结束时间
        :param location: 可选，地点
        :param status: 可选，状态
        :param reminders: 可选，提醒
        :param content: 可选，备注
        """
        data = optionaldict(
            title=title,
            startDate=start_date,
            endDate=end_date,
            location=location,
            status=status,
            reminders=reminders,
            content=content
        )
        return self._put(
            'api/events/{0}'.format(id),
            data=data
        )

    def like(self, id):
        """
        赞日程

        详情请参考
        http://docs.teambition.com/wiki/events#events-like

        :param id: 日程 ID
        :return: 返回的 JSON 数据包
        """
        return self._post('api/events/{0}/like'.format(id))

    def archive(self, id):
        """
        归档日程

        详情请参考
        http://docs.teambition.com/wiki/events#events-archive

        :param id: 日程 ID
        :return: 返回的 JSON 数据包
        """
        return self._post('api/events/{0}/archive'.format(id))

    def unarchive(self, id):
        """
        取消归档日程

        详情请参考
        http://docs.teambition.com/wiki/events#events-unarchive

        :param id: 日程 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/events/{0}/archive'.format(id))

    def update_tags(self, id, tag_ids):
        """
        更新日程标签

        详情请参考
        http://docs.teambition.com/wiki/events#events-update-tags

        :param id: 日程 ID
        :param tag_ids: 标签 ID 列表
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/events/{0}/tagIds'.format(id),
            data={
                'tagIds': tag_ids
            }
        )

    def get_tags(self, id):
        """
        获取日程标签列表

        :param id: 日程 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/events/{0}/tags'.format(id))

    def create_tag(self, id, name):
        """
        新建标签

        :param id: 日程 ID
        :param name: 标签名称
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'api/events/{0}/tags'.format(id),
            data={
                'name': name
            }
        )

    def add_tag(self, id, tag_id):
        """
        关联标签

        :param id: 日程 ID
        :param tag_id: 标签 ID
        :return: 返回的 JSON 数据包
        """
        return self._put('api/events/{0}/tags/{1}'.format(id, tag_id))

    def get_objectlinks(self, id):
        """
        获取日程关联的 objectlink 列表

        :param id: 日程 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/events/{0}/objectlinks'.format(id))

    def create_objectlink(self, id, linked_id, linked_type):
        """
        关联对象

        :param id: 日程 ID
        :param linked_id: 关联对象 ID
        :param linked_type: 关联对象类型
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'api/objectlinks',
            data={
                '_parentId': id,
                'parentType': 'event',
                '_linkedId': linked_id,
                'linkedType': linked_type
            }
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from optionaldict import optionaldict

from teambition.api.base import TeambitionAPI


class StageTemplates(TeambitionAPI):

    def get(self):
        """
        获取用户自定义的阶段模板

        详情请参考
        http://docs.teambition.com/wiki/stages-template#stages-template-get

        :return: 返回的 JSON 数据包
        """
        return self._get('api/stagetemplates')

    def create(self, title, stages):
        """
        新建模板

        详情请参考
        http://docs.teambition.com/wiki/stages-template#stages-template-create

        :param title: 模板标题
        :param stages: 阶段名称列表，至少两个阶段
        :return: 返回的 JSON 数据包
        """
        if isinstance(stages, (tuple, list)):
            stages = ','.join(stages)
        return self._post(
            'api/stagetemplates',
            data={
                'title': title,
                'stages': stages
            }
        )

    def delete(self, id):
        """
        删除模板

        详情请参考
        http://docs.teambition.com/wiki/stages-template#stages-template-delete

        :param id: 路径参数
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/stagetemplates/{0}'.format(id))

    def update(self, id, title=None, stages=None):
        """
        更新模板

        详情请参考
        http://docs.teambition.com/wiki/stages-template#stages-template-update

        :param id: 路径参数
        :param title: 可选，模板标题
        :param stages: 可选，阶段名称列表，至少两个阶段
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            title=title,
            stages=stages
        )
        return self._put(
            'api/stagetemplates/{0}'.format(id),
            data=data
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from teambition.api.oauth import OAuth  # NOQA
from teambition.api.projects import Projects  # NOQA
from teambition.api.tasklists import Tasklists  # NOQA
from teambition.api.stages import Stages  # NOQA
from teambition.api.tasks import Tasks  # NOQA
from teambition.api.users import Users  # NOQA
from teambition.api.organizations import Organizations  # NOQA
from teambition.api.stagetemplates import StageTemplates  # NOQA
from teambition.api.teams import Teams  # NOQA
from teambition.api.subtasks import Subtasks  # NOQA
from teambition.api.messages import Messages  # NOQA
from teambition.api.posts import Posts  # NOQA
from teambition.api.collections import Collections  # NOQA
from teambition.api.works import Works  # NOQA
from teambition.api.events import Events  # NOQA
from teambition.api.tags import Tags  # NOQA
from teambition.api.objectlinks import ObjectLinks  # NOQA
from teambition.api.activities import Activities  # NOQA
from teambition.api.webhooks import Webhooks  # NOQA

__all__ = [
    'OAuth',
    'Projects',
    'Tasklists',
    'Stages',
    'Tasks',
    'Users',
    'Organizations',
    'StageTemplates',
    'Teams',
    'Subtasks',
    'Messages',
    'Posts',
    'Collections',
    'Works',
    'Events',
    'Tags',
    'ObjectLinks',
    'Activities',
]



#!/usr/bin/env python
from __future__ import with_statement
import os
from setuptools import setup, find_packages

readme = 'README.md'
if os.path.exists('README.rst'):
    readme = 'README.rst'
with open(readme) as f:
    long_description = f.read()

with open('requirements.txt') as f:
    requirements = [l for l in f.read().splitlines() if l]

setup(
    name='teambition',
    version='0.1.6',
    author='messense',
    author_email='messense@icloud.com',
    url='https://github.com/messense/teambition-api',
    packages=find_packages(),
    keywords='Teambition, SDK',
    description='Teambition Python API',
    long_description=long_description,
    install_requires=requirements,
    include_package_data=True,
    tests_require=['nose', 'httmock'],
    test_suite='nose.collector',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS',
        'Operating System :: POSIX',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from optionaldict import optionaldict

from teambition.api.base import TeambitionAPI


class Tasklists(TeambitionAPI):

    def get(self, id=None, project_id=None):
        """
        获取任务分组列表

        详情请参考
        http://docs.teambition.com/wiki/tasklists#tasklists-get

        :param id: 可选，任务分组 ID
        :param project_id: 可选，项目 ID
        :return: 返回的 JSON 数据包
        """
        assert id or project_id
        if id:
            endpoint = 'api/tasklists/{0}'.format(id)
        elif project_id:
            endpoint = 'api/projects/{0}/tasklists'.format(project_id)
        return self._get(endpoint)

    def create(self, project_id, title, description=None):
        """
        新建任务分组

        详情请参考
        http://docs.teambition.com/wiki/tasklists#tasklists-create

        :param project_id: 项目 ID
        :param title: 任务列表标题
        :param description: 可选，任务列表描述
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            _projectId=project_id,
            title=title,
            description=description
        )
        return self._post(
            'api/tasklists',
            data=data
        )

    def delete(self, id):
        """
        删除任务分组

        详情请参考
        http://docs.teambition.com/wiki/tasklists#tasklists-delete

        :param id: 任务分组 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/tasklists/{0}'.format(id))

    def update(self, id, title=None, description=None, is_archived=None):
        """
        更新任务分组

        详情请参考
        http://docs.teambition.com/wiki/tasklists#tasklists-update

        :param id: 任务分组 ID
        :param title: 可选，标题
        :param description: 可选，描述
        :param is_archived: 可选，是否归档
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            title=title,
            description=description,
            isArchived=is_archived
        )
        return self._put(
            'api/tasklists/{0}'.format(id),
            data=data
        )

    def update_stage_ids(self, id, stage_ids):
        """
        更新任务分组内阶段顺序

        详情请参考
        http://docs.teambition.com/wiki/tasklists#tasklists-update-stageids

        :param id: 任务分组 ID
        :param stage_ids: 阶段 ID 数组
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/tasklists/{0}/stageIds'.format(id),
            data={
                'stageIds': stage_ids
            }
        )

    def archive(self, id):
        """
        任务分组存档

        详情请参考
        http://docs.teambition.com/wiki/tasklists#tasklists-archive

        :param id: 任务分组 ID
        :return: 返回的 JSON 数据包
        """
        return self._post('api/tasklists/{0}/archive'.format(id))

    def unarchive(self, id):
        """
        任务分组取消存档

        详情请参考
        http://docs.teambition.com/wiki/tasklists#tasklists-unarchive

        :param id: 任务分组 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/tasklists/{0}/archive'.format(id))

    def get_tasks(self, id, executor_id=None, is_done=False, dump_type='json',
                  all=False, page=1, limit=30, **kwargs):
        """
        获取任务分组任务列表

        详情请参考
        http://docs.teambition.com/wiki/tasklists#tasklists-get-tasks

        :param id: 任务分组 ID
        :param executor_id: 可选，执行者 ID
        :param is_done: 可选，是否已完成，默认为 False
        :param dump_type: 可选，默认为 json，支持 excel
        :param all: 可选，是否所有类型，包含完成和未完成
        :param page: 可选，页码，默认为 1
        :param limit: 可选，每页数量，默认为 30，最大值为 1000
        :return: 返回的 JSON  数据包
        """
        params = optionaldict(
            isDone=is_done,
            _executorId=executor_id,
            dumpType=dump_type,
            all=all,
            page=page,
            limit=limit,
            **kwargs
        )
        return self._get(
            'api/tasklists/{0}/tasks'.format(id),
            params=params
        )

    def get_stages(self, id):
        """
        获取任务分组的阶段列表

        详情请参考
        http://docs.teambition.com/wiki/stages#stages-get

        :param tasklist_id: 任务分组 ID
        :return: 返回的 JSON 数据包
        """
        return self._get(
            'api/stages',
            params={
                '_tasklistId': tasklist_id
            }
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from optionaldict import optionaldict

from teambition.api.base import TeambitionAPI


class Collections(TeambitionAPI):

    def get(self, id=None, parent_id=None):
        """
        获取文件集

        详情请参考
        http://docs.teambition.com/wiki/collections#collections-get

        :param id: 可选，文件集 ID
        :param parent_id: 可选，父级 ID
        :return: 返回的 JSON 数据包
        """
        params = {}
        if id:
            endpoint = 'api/collections/{0}'.format(id)
        elif parent_id:
            endpoint = 'api/collections'
            params['_parentId'] = parent_id
        return self._get(endpoint, params=params)

    def create(self, title, project_id, parent_id=None, description=None,
               color=None):
        """
        新建文件集

        详情请参考
        http://docs.teambition.com/wiki/collections#collections-create

        :param title: 文件集标题
        :param project_id: 项目 ID
        :param parent_id: 可选，父级 ID，默认为项目 ID
        :param description: 可选，描述
        :param color: 可选，颜色，默认为空
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            title=title,
            _projectId=project_id,
            _parentId=parent_id or project_id,
            description=description,
            color=color
        )
        return self._post(
            'api/collections',
            data=data
        )

    def delete(self, id):
        """
        删除文件集

        详情请参考
        http://docs.teambition.com/wiki/collections#collections-delete

        :param id: 文件集 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/collections/{0}'.format(id))

    def update(self, id, title=None, parent_id=None, description=None,
               color=None):
        """
        更新文件集

        详情请参考
        http://docs.teambition.com/wiki/collections#collections-update

        :param id: 文件集 ID
        :param title: 可选，文件集标题
        :param parent_id: 可选，父级 ID
        :param description: 可选，描述
        :param color: 可选，颜色，默认为空
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            title=title,
            _parentId=parent_id,
            description=description,
            color=color,
            isArchived=is_archived
        )
        return self._put(
            'api/collections/{0}'.format(id),
            data=data
        )

    def archive(self, id):
        """
        归档文件集

        详情请参考
        http://docs.teambition.com/wiki/collections#collections-archive

        :param id: 文件集 ID
        :return: 返回的 JSON 数据包
        """
        return self._post('api/collections/{0}/archive'.format(id))

    def unarchive(self, id):
        """
        归档文件集

        详情请参考
        http://docs.teambition.com/wiki/collections#collections-unarchive

        :param id: 文件集 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/collections/{0}/archive'.format(id))

    def move(self, id, parent_id):
        """
        移动文件集

        详情请参考
        http://docs.teambition.com/wiki/collections#collections-move

        :param id: 文件集 ID
        :param parent_id: 新的父级目录 ID
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/collections/{0}/move'.format(id),
            data={
                '_parentId': parent_id
            }
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from optionaldict import optionaldict

from teambition.api.base import TeambitionAPI


class Tasks(TeambitionAPI):

    def get(self, id, detail_type=None):
        """
        获取任务

        详情请参考
        http://docs.teambition.com/wiki/tasks#tasks-get-by-id

        :param id: 任务 ID
        :param detail_type: 可选值complete, 则会包含子任务详细信息
        :return: 返回的 JSON 数据包
        """
        params = optionaldict(
            detailType=detail_type
        )
        return self._get(
            'api/tasks/{0}'.format(id),
            params=params
        )

    def create(self, content, tasklist_id, stage_id=None, executor_id=None,
               involve_members=None, due_date=None, priority=None,
               recurrence=None, tag_ids=None):
        """
        创建任务

        详情请参考
        http://docs.teambition.com/wiki/tasks#tasks-create

        :param content: 任务内容
        :param tasklist_id: 任务分组 ID
        :param stage_id: 可选，阶段 ID，默认为任务分组的第一个阶段
        :param executor_id: 可选，执行者 ID，默认为空
        :param involve_members: 可选，参与者 ID 数组，默认为创建者和执行者
        :param due_date: 可选，截止日期
        :param priority: 可选，优先级，可选值为 0，1，2，对应普通、紧急、非常紧急
        :param recurrence: 可选，重复规则列表
        :param tag_ids: 可选，标签 ID 列表
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            content=content,
            _tasklistId=tasklist_id,
            _stageId=stage_id,
            _executorId=executor_id,
            involveMembers=involve_members,
            dueDate=due_date,
            priority=priority,
            recurrence=recurrence,
            tagIds=tag_ids
        )
        return self._post(
            'api/tasks',
            data=data
        )

    def delete(self, id):
        """
        删除任务

        详情请参考
        http://docs.teambition.com/wiki/tasks#tasks-delete

        :param id: 任务 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/tasks/{0}'.format(id))

    def update(self, id, content=None, tasklist_id=None, stage_id=None,
               executor_id=None, involve_members=None, due_date=None,
               priority=None, recurrence=None, is_done=None, note=None):
        """
        更新任务

        详情请参考
        http://docs.teambition.com/wiki/tasks#tasks-update

        :param content: 任务内容
        :param tasklist_id: 任务分组 ID
        :param stage_id: 可选，阶段 ID，默认为任务分组的第一个阶段
        :param executor_id: 可选，执行者 ID，默认为空
        :param involve_members: 可选，参与者 ID 数组，默认为创建者和执行者
        :param due_date: 可选，截止日期
        :param priority: 可选，优先级，可选值为 0，1，2，对应普通、紧急、非常紧急
        :param recurrence: 可选，重复规则列表
        :param is_done: 可选，完成状态
        :param note: 可选，笔记
        """
        data = optionaldict(
            content=content,
            _tasklistId=tasklist_id,
            _stageId=stage_id,
            _executorId=executor_id,
            involveMembers=involve_members,
            dueDate=due_date,
            priority=priority,
            recurrence=recurrence,
            isDone=is_done,
            note=note
        )
        return self._put(
            'api/tasks/{0}'.format(id),
            data=data
        )

    def like(self, id):
        """
        赞任务

        详情请参考
        http://docs.teambition.com/wiki/tasks#tasks-like

        :param id: 任务 ID
        :return: 返回的 JSON 数据包
        """
        return self._post('api/tasks/{0}/like'.format(id))

    def fork(self, id, stage_id, do_link=False, do_linked=False):
        """
        复制任务

        详情请参考
        http://docs.teambition.com/wiki/tasks#tasks-fork

        :param id: 任务 ID，任务 ID
        :param stage_id: 目的阶段 ID
        :param do_link: 可选，是否关联复制出的任务，默认为 False
        :param do_linked: 可选，是否会被复制出的任务关联，默认为 False
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            _stageId=stage_id,
            doLink=do_link,
            doLinked=do_linked
        )
        return self._put(
            'api/tasks/{0}/fork'.format(id),
            data=data
        )

    def move(self, id, stage_id):
        """
        移动任务

        详情请参考
        http://docs.teambition.com/wiki/tasks#tasks-move

        :param id: 任务 ID
        :param stage_id: 目的阶段 ID
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/tasks/{0}/move'.format(id),
            data={
                '_stageId': stage_id
            }
        )

    def archive(self, id):
        """
        归档任务

        详情请参考
        http://docs.teambition.com/wiki/tasks#tasks-archive

        :param id: 任务 ID
        :return: 返回的 JSON 数据包
        """
        return self._post('api/tasks/{0}/archive'.format(id))

    def unarchive(self, id):
        """
        取消归档任务

        详情请参考
        http://docs.teambition.com/wiki/tasks#tasks-unarchive

        :param id: 任务 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/tasks/{0}/archive'.format(id))

    def update_subtasks(self, id, subtask_ids):
        """
        更新任务内子任务顺序

        详情请参考
        http://docs.teambition.com/wiki/tasks#tasks-update-subtaskids

        :param id: 任务 ID
        :param subtask_ids: 子任务 ID 列表
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/tasks/{0}/subtaskIds'.format(id),
            data={
                'subtaskIds': subtask_ids
            }
        )

    def update_tags(self, id, tag_ids):
        """
        更新任务标签

        详情请参考
        http://docs.teambition.com/wiki/tasks#tasks-update-tags

        :param id: 任务 ID
        :param tag_ids: 标签 ID 列表
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/tasks/{0}/tagIds'.format(id),
            data={
                'tagIds': tag_ids
            }
        )

    def update_content(self, id, content):
        """
        更新任务内容

        详情请参考
        http://docs.teambition.com/wiki/tasks#tasks-update_content

        :param id: 任务 ID
        :param content: 任务内容
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/tasks/{0}/content'.format(id),
            data={
                'content': content
            }
        )

    def update_note(self, id, note):
        """
        更新任务备注

        详情请参考
        http://docs.teambition.com/wiki/tasks#tasks-update_note

        :param id: 任务 ID
        :param note: 任务备注
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/tasks/{0}/note'.format(id),
            data={
                'note': note
            }
        )

    def update_executor(self, id, executor_id):
        """
        更新任务执行者

        详情请参考
        http://docs.teambition.com/wiki/tasks#tasks-update_executor

        :param id: 任务 ID
        :param executor_id: 执行者 ID
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/tasks/{0}/_executorId'.format(id),
            data={
                '_executorId': executor_id
            }
        )

    def update_status(self, id, is_done):
        """
        更新任务状态

        详情请参考
        http://docs.teambition.com/wiki/tasks#tasks-update_status

        :param id: 任务 ID
        :param is_done: 是否已经完成
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/tasks/{0}/isDone'.format(id),
            data={
                'isDone': is_done
            }
        )

    def update_duedate(self, id, duedate):
        """
        更新截止日期

        详情请参考
        http://docs.teambition.com/wiki/tasks#tasks-update_duedate

        :param id: 任务 ID
        :param duedate: 截止日期，请使用 ISOString 格式，置空请传 None
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/tasks/{0}/dueDate'.format(id),
            data={
                'dueDate': duedate
            }
        )

    def get_subtasks(self, id):
        """
        获取子任务列表

        :param id: 任务 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/tasks/{0}/subtasks'.format(id))

    def get_tags(self, id):
        """
        获取任务标签列表

        :param id: 任务 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/tasks/{0}/tags'.format(id))

    def create_tag(self, id, name):
        """
        新建标签

        :param id: 任务 ID
        :param name: 标签名称
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'api/tasks/{0}/tags'.format(id),
            data={
                'name': name
            }
        )

    def add_tag(self, id, tag_id):
        """
        关联标签

        :param id: 任务 ID
        :param tag_id: 标签 ID
        :return: 返回的 JSON 数据包
        """
        return self._put('api/tasks/{0}/tags/{1}'.format(id, tag_id))

    def get_objectlinks(self, id):
        """
        获取任务关联的 objectlink 列表

        :param id: 任务 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/tasks/{0}/objectlinks'.format(id))

    def create_objectlink(self, id, linked_id, linked_type):
        """
        关联对象

        :param id: 任务 ID
        :param linked_id: 关联对象 ID
        :param linked_type: 关联对象类型
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'api/objectlinks',
            data={
                '_parentId': id,
                'parentType': 'task',
                '_linkedId': linked_id,
                'linkedType': linked_type
            }
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from optionaldict import optionaldict

from teambition.api.base import TeambitionAPI


class Tags(TeambitionAPI):

    def get(self, id):
        """
        获取标签信息

        详情请参考
        http://docs.teambition.com/wiki/tags#tags-get

        :param id: 标签 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/tags/{0}'.format(id))

    def create(self, name, project_id):
        """
        新建标签

        详情请参考
        http://docs.teambition.com/wiki/tags#tags-create

        :param name: 标签名称
        :param project_id: 项目 ID
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'api/tags',
            data={
                'name': name,
                '_projectId': project_id
            }
        )

    def delete(self, id):
        """
        删除标签

        详情请参考
        http://docs.teambition.com/wiki/tags#tags-delete

        :param id: 标签 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/tags/{0}'.format(id))

    def update(self, id, name, color=None):
        """
        更新标签

        详情请参考
        http://docs.teambition.com/wiki/tags#tags-update

        :param id: 标签 ID
        :param name: 标签名称
        :param color: 可选，颜色标记
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            name=name,
            color=color
        )
        return self._put(
            'api/tags/{0}'.format(id),
            data=data
        )

    def archive(self, id):
        """
        归档标签

        详情请参考
        http://docs.teambition.com/wiki/tags#tags-archive

        :param id: 标签 ID
        :return: 返回的 JSON 数据包
        """
        return self._post('api/tags/{0}/archive'.format(id))

    def unarchive(self, id):
        """
        取消归档标签

        详情请参考
        http://docs.teambition.com/wiki/tags#tags-unarchive

        :param id: 标签 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/tags/{0}/archive'.format(id))

    def get_tasks(self, id):
        """
        获取标签关联的任务

        :param id: 标签 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/tags/{0}/tasks'.format(id))

    def get_posts(self, id):
        """
        获取标签关联的分享

        :param id: 标签 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/tags/{0}/posts'.format(id))

    def get_events(self, id):
        """
        获取标签关联的日程

        :param id: 标签 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/tags/{0}/events'.format(id))

    def get_works(self, id):
        """
        获取标签关联的文件

        :param id: 标签 ID
        :return: 返回的 JSON 数据包
        """
        return self._get('api/tags/{0}/works'.format(id))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import requests
from furl import furl

from teambition.api.base import TeambitionAPI


class OAuth(TeambitionAPI):

    def get_authorize_url(self, redirect_uri, state='', lang='zh'):
        """
        获取授权地址

        详情请参考
        http://docs.teambition.com/wiki/oauth2#oauth2-authorize

        :param redirect_uri: 授权回调地址
        :param state: 原样返回给客户端
        :param lang: 语言类型，可选 zh, en，默认为 zh
        :return: 授权地址
        """
        oauth2_url = furl('https://account.teambition.com/oauth2/authorize')
        oauth2_url.add(args={
            'client_id': self.client_id,
            'redirect_uri': redirect_uri,
            'state': state,
            'lang': lang
        })
        return oauth2_url.url

    def fetch_access_token(self, code, grant_type='code'):
        """
        根据 code 获取 access_token

        详情请参考
        http://docs.teambition.com/wiki/oauth2#oauth2-access_token

        :param code: 授权完成返回的 code 参数值
        :param grant_type: 固定值为 code
        :return: access_token 值
        """
        res = self._post(
            'oauth2/access_token',
            data={
                'client_id': self.client_id,
                'client_secret': self.client_secret,
                'code': code,
                'grant_type': grant_type
            }
        )
        token = res['access_token']
        self._client._access_token = token
        return token

    get_access_token = fetch_access_token  # Alias

    def check(self, access_token=None):
        """
        验证 access_token 是否合法

        详情请参考
        http://docs.teambition.com/wiki/oauth2#oauth2-check

        :param access_token: 可选，access_token
        :return: 返回的 JSON 数据包
        """
        token = access_token or self.access_token
        headers = {
            'Authorization': 'OAuth2 {0}'.format(token),
        }
        valid = True
        try:
            self._get(
                'api/applications/{0}/tokens/check'.format(self.client_id),
                headers=headers
            )
        except requests.HTTPError:
            valid = False

        return valid



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from optionaldict import optionaldict

from teambition.api.base import TeambitionAPI


class Subtasks(TeambitionAPI):

    def get(self, id=None, task_id=None):
        """
        获取子任务，子任务 ID 和任务 ID 参数只能有一个存在

        详情请参考
        http://docs.teambition.com/wiki/tasks-subtasks#tasks-subtasks-get

        :param id: 可选，子任务 ID
        :param task_id: 可选，任务 ID
        :return: 返回的 JSON 数据包
        """
        assert id or task_id, 'id 和 task_id 必须提供其中之一'
        if id:
            endpoint = 'api/subtasks/{0}'.format(id)
        elif task_id:
            endpoint = 'api/tasks/{0}/subtasks'.format(task_id)
        return self._get(endpoint)

    def create(self, content, task_id, executor_id=None):
        """
        新建子任务

        详情请参考
        http://docs.teambition.com/wiki/tasks-subtasks#tasks-subtasks-create

        :param content: 子任务内容
        :param task_id: 任务 ID
        :param executor_id: 可选，执行者 ID，默认为空
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            content=content,
            _taskId=task_id,
            _executorId=executor_id
        )
        return self._post(
            'api/subtasks',
            data=data
        )

    def delete(self, id):
        """
        删除子任务

        详情请参考
        http://docs.teambition.com/wiki/tasks-subtasks#tasks-subtasks-delete

        :param id: 子任务 ID
        :return: 返回的 JSON 数据包
        """
        return self._delete('api/subtasks/{0}'.format(id))

    def update(self, id, content=None, is_done=None, executor_id=None):
        """
        更新子任务

        详情请参考
        http://docs.teambition.com/wiki/tasks-subtasks#tasks-subtasks-update

        :param id: 子任务 ID
        :param content: 可选，子任务内容
        :param is_done: 可选，子任务完成状态
        :param executor_id: 可选，执行者 ID
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            content=content,
            isDone=is_done,
            _executorId=executor_id
        )
        return self._put(
            'api/subtasks/{0}'.format(id),
            data=data
        )

    def transform(self, id, do_link=False, do_linked=False):
        """
        转换成任务

        详情请参考
        http://docs.teambition.com/wiki/tasks-subtasks#tasks-subtasks-transform

        :param id: 子任务 ID
        :param do_link: 是否关联转出的任务，默认为 False
        :param do_linked: 是否会被转出的任务关联，默认为 False
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            doLink=do_link,
            doLinked=do_linked
        )
        return self._put(
            'api/subtasks/{0}/transform',
            data=data
        )

    def update_content(self, id, content):
        """
        更新子任务内容

        详情请参考
        http://docs.teambition.com/wiki/tasks-subtasks#tasks-subtasks-update_content

        :param id: 子任务 ID
        :param content: 任务内容
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/subtasks/{0}/content'.format(id),
            data={
                'content': content
            }
        )

    def update_executor(self, id, executor_id):
        """
        更新子任务执行者

        详情请参考
        http://docs.teambition.com/wiki/tasks-subtasks#tasks-subtasks-update_executor

        :param id: 子任务 ID
        :param executor_id: 执行者 ID
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/subtasks/{0}/_executorId'.format(id),
            data={
                '_executorId': executor_id
            }
        )

    def update_status(self, id, is_done):
        """
        更新子任务状态

        详情请参考
        http://docs.teambition.com/wiki/tasks-subtasks#tasks-subtasks-update_status

        :param id: 子任务 ID
        :param is_done: 是否已经完成
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/subtasks/{0}/isDone'.format(id),
            data={
                'isDone': is_done
            }
        )

    def update_duedate(self, id, duedate):
        """
        更新截止日期

        详情请参考
        http://docs.teambition.com/wiki/tasks-subtasks#tasks-subtasks-update_duedate

        :param id: 子任务 ID
        :param duedate: 截止日期，请使用 ISOString 格式，置空请传 None
        :return: 返回的 JSON 数据包
        """
        return self._put(
            'api/subtasks/{0}/dueDate'.format(id),
            data={
                'dueDate': duedate
            }
        )
